from .regression import find_beta_WMCC, find_beta_OLS, find_beta_WLS
from .dictionary import harmonic_dictionary
from .generator import contaminate_time_series, irregular_sampling, trigonometric_model
from .periodogram import periodogram

__version__ = '0.5'
